/*
    Game Genie Decoder Program v 3.2
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 5/27/01; 8:45:10 PM

#include "ggdecode.h"

#ifdef _GENESIS_SUPPORT_

#include <string.h>
#include <dialogs.h>
#include <stdio.h>
#include <alloc.h>

short int decodeGenesisToHex(const char digit) {
	short int value;
	
	switch (digit) {
		case 'A': value = 0; break;
		case 'B': value = 1; break;
		case 'C': value = 2; break;
		case 'D': value = 3; break;
		case 'E': value = 4; break;
		case 'F': value = 5; break;
		case 'G': value = 6; break;
		case 'H': value = 7; break;
		case 'J': value = 8; break;
		case 'K': value = 9; break;
		case 'L': value = 10; break;
		case 'M': value = 11; break;
		case 'N': value = 12; break;
		case 'P': value = 13; break;
		case 'R': value = 14; break;
		case 'S': value = 15; break;
		case 'T': value = 16; break;
		case 'V': value = 17; break;
		case 'W': value = 18; break;
		case 'X': value = 19; break;
		case 'Y': value = 20; break;
		case 'Z': value = 21; break;
		case '0': value = 22; break;
		case '1': value = 23; break;
		case '2': value = 24; break;
		case '3': value = 25; break;
		case '4': value = 26; break;
		case '5': value = 27; break;
		case '6': value = 28; break;
		case '7': value = 29; break;
		case '8': value = 30; break;
		case '9': value = 31; break;
		default: value = -1; break;
	}
	
	return value;
}

short int isValidGenesis(unsigned char *ggcode, short int *length, char *bitstr) {
	short int loop, digit;
	const char *hexbits[] = {
		"00000","00001","00010","00011","00100","00101","00110","00111",
		"01000","01001","01010","01011","01100","01101","01110","01111",
		"10000","10001","10010","10011","10100","10101","10110","10111",
		"11000","11001","11010","11011","11100","11101","11110","11111"
	};
	
	if (*length == 9) {
		if (ggcode[4] != '-' && ggcode[4] != 173) {
			return -1;
		}
		
		for (loop = 4; loop < *length; loop++) {
			ggcode[loop] = ggcode[loop + 1];
		}
		
		ggcode[--(*length)] = 0;		
	} else if (*length != 8) {
		return -1;
	}
	
	for (loop = 0; loop < *length; loop++) {
		if ((digit = decodeGenesisToHex(ggcode[loop])) != -1) {
			strcat(bitstr,hexbits[digit]);
		} else {
			return -2;
		}
	}
	
	return 1;
}

void decodeGenesis(const char *bitstr, char *addressbits, char *valuebits) {
	char str[41];
	
	// make a copy of the bit string
	strcpy(str,bitstr);
	
	str[24] = 0;
	strcat(addressbits,str+16);
	str[24] = bitstr[24];
	
	str[16] = 0;
	strcat(addressbits,str+8);
	str[16] = bitstr[16];
	
	str[40] = 0;
	strcat(addressbits,str+32);
	
	str[32] = 0;
	strcat(valuebits,str+29);
	str[32] = bitstr[32];
	
	str[29] = 0;
	strcat(valuebits,str+24);
	str[29] = bitstr[29];
	
	str[8] = 0;
	strcat(valuebits,str);
}

void convertGenesis(char *ggcode) {
	short int length = strlen(ggcode), result;
	char str[30],saveCode[13],bitstr[41],addressbits[25],valuebits[17];
	unsigned long int address;
	unsigned short int value;
	HANDLE dlg;
	
	// preserve the code
	strcpy(saveCode,ggcode);
	
	// erase bit strings
	bitstr[0] = 0;
	addressbits[0] = 0;
	valuebits[0] = 0;
	
	result = isValidGenesis(ggcode,&length,bitstr);
	
	if (result == -1) {
		DlgMessage((char *)errors[INVALID_LENGTH],"Genesis Codes must be 8 characters!",BT_OK,BT_NONE);
	} else if (result == -2) {
		DlgMessage((char *)errors[INVALID_CODE],"Only ABCDEFGHJKLMNPRSTVWXYZ0123456789 are valid!",BT_OK,BT_NONE);
	} else {
		if ((dlg = DialogNewSimple(GENESIS_WIDTH,GENESIS_HEIGHT)) != H_NULL) {
			decodeGenesis(bitstr,addressbits,valuebits);
			address = toDecimal(addressbits,2);
			value = (unsigned short int)toDecimal(valuebits,2);
			
			DialogAddTitle(dlg,titles[GENESIS],BT_OK,BT_NONE);
			
			sprintf(str,ggstr,saveCode);
			DialogAddText(dlg,5,15,str);
			
			sprintf(str,"Value: %04hXh, Address: %06lXh",value,address);
			DialogAddText(dlg,5,25,str);
			
			DialogDo(dlg,CENTER,CENTER,NULL,NULL);
			
			HeapFree(dlg);
		} else {
			DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		}
	}
}

#endif
