/*
    Game Genie Decoder Program v 3.2
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 5/26/2001; 3:08:04 AM

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include "ggdecode.h"

#include <string.h>
#include <ctype.h>
#include <math.h>
#include <dialogs.h>
#include <menus.h>
#include <alloc.h>
#include <kbd.h>

#ifdef _TI89_
short _ti89;                  // Produce .89Z File
#endif

#ifdef _TI92_
short _ti92plus;              // Produce .9XZ File
#endif

const char *errors[] = {
	"Error: Invalid Length",
	"Error: Insufficient Memory",
	"Not enough free memory!",
	"Error: Invalid Code"
};

const char *titles[] = {
	"Game Genie Decoder v3.2",
	"NES Decoder",
	"SNES Decoder",
	"Genesis Decoder",
	"Game Boy/Game Gear Decoder"
};

const char *ggstr = "Game Genie Code: %s";

void strupr(char *str) {
	short int length = strlen(str), loop;
	
	for (loop = 0; loop < length; loop++) {
		str[loop] = toupper(str[loop]);
	}
}

unsigned short int convertHexDec(const char hex) {
	if (hex >= '0' && hex <= '9') {
		return (hex - '0');
	} else if (hex >= 'a' && hex <= 'f') {
		return (hex - 'a') + 10;
	} else if (hex >= 'A' && hex <= 'F') {
		return (hex - 'A') + 10;
	} else {
		return -1;
	}
}

unsigned long int toDecimal(const char *str, short int base) {
	short int length = strlen(str), pos, place = 0;
	unsigned long int decimal = 0;
	
	for (pos = length - 1; pos >= 0; pos--) {
		decimal += convertHexDec(str[pos]) * (unsigned long int)pow(base,place);
		place++;
	}
	
	return decimal;
}

// Main Function
void _main(void)
{
	HANDLE dlg = H_NULL, system = H_NULL;
	short int systype = 0;
	char ggcode[13];
	
	// allocate memory for the dialog box
	if ((dlg = DialogNewSimple(MAIN_WIDTH,MAIN_HEIGHT)) == H_NULL) {
		DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		return;
	}
	
	// allocate memory for the pulldown menu
	if ((system = PopupNew(NULL,0)) == H_NULL) {
		DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		HeapFree(dlg);
		return;
	}
	
#ifdef _NES_SUPPORT_
	PopupAddText(system,-1,"NES",NES);
	systype = NES;
#endif

#ifdef _SNES_SUPPORT_
	PopupAddText(system,-1,"Super NES",SNES);
	
	if (systype == 0) {
		systype = SNES;
	}
#endif

#ifdef _GENESIS_SUPPORT_
	PopupAddText(system,-1,"Genesis",GENESIS);
	
	if (systype == 0) {
		systype = GENESIS;
	}
#endif

#ifdef _GBGG_SUPPORT_
	PopupAddText(system,-1,"Game Boy/Game Gear",GBGG);
	
	if (systype == 0) {
		systype = GBGG;
	}
#endif

	// design the dialog box
	DialogAddTitle(dlg,titles[0],BT_OK,BT_CANCEL);
	DialogAddPulldown(dlg,5,15,"System: ",system,0);
	DialogAddRequest(dlg,5,25,"Code: ",0,11,12);
	
	do {
		// truncate the string text
		ggcode[0] = 0;
		
		if (DialogDo(dlg,CENTER,CENTER,ggcode,&systype) == KEY_ENTER) {
			strupr(ggcode);
			
			if (systype == NES) {
#ifdef _NES_SUPPORT_			
				convertNES(ggcode);
#endif				
			} else if (systype == SNES) {
#ifdef _SNES_SUPPORT_			
				convertSNES(ggcode);
#endif				
			} else if (systype == GENESIS) {
#ifdef _GENESIS_SUPPORT_			
				convertGenesis(ggcode);
#endif				
			} else if (systype == GBGG) {
#ifdef _GBGG_SUPPORT_
				convertGBGG(ggcode);
#endif
			}
		}
	} while (DlgMessage((char *)titles[0],"Decode More?",BT_YES,BT_NO) == KEY_ENTER);

	HeapFree(system);
	HeapFree(dlg);
	
	if ((dlg = DialogNewSimple(COPY_WIDTH,COPY_HEIGHT)) != H_NULL) {
		DialogAddTitle(dlg,titles[0],BT_OK,BT_NONE);
		DialogAddText(dlg,5,15,"Copyright (C) 2001 John David Ratliff");
		DialogAddText(dlg,5,25,"E-Mail: jdratlif@cs.indiana.edu");
		DialogAddText(dlg,5,35,"URL: http://www.technoplaza.net");
		DialogDo(dlg,CENTER,CENTER,NULL,NULL);
		HeapFree(dlg);
	}
}
