/*
    Game Genie Decoder Program v 3.2
    Copyright (C) 2001 John David Ratliff
    Questions or Comments: 
    	E-Mail: jdratlif@cs.indiana.edu
    	URL:	http://www.technoplaza.net 
    	
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// C Source File
// Created 5/27/01; 8:23:40 PM

#include "ggdecode.h"

#ifdef _SNES_SUPPORT_

#include <string.h>
#include <dialogs.h>
#include <stdio.h>
#include <alloc.h>

char decodeSNEStoHex(const char digit) {
	char decoded;
	
	switch (digit) {
		case 'D': decoded = '0'; break;
		case 'F': decoded = '1'; break;
		case '4': decoded = '2'; break;
		case '7': decoded = '3'; break;
		case '0': decoded = '4'; break;
		case '9': decoded = '5'; break;
		case '1': decoded = '6'; break;
		case '5': decoded = '7'; break;
		case '6': decoded = '8'; break;
		case 'B': decoded = '9'; break;
		case 'C': decoded = 'A'; break;
		case '8': decoded = 'B'; break;
		case 'A': decoded = 'C'; break;
		case '2': decoded = 'D'; break;
		case '3': decoded = 'E'; break;
		case 'E': decoded = 'F'; break;
		default: decoded = 0; break;
	}
	
	return decoded;
}

short int isValidSNES(unsigned char *ggcode, short int *length) {
	short int loop;
	
	if (*length == 9) {
		if (ggcode[4] != '-' && ggcode[4] != 173) {
			return -1;
		}
		
		for (loop = 4; loop < *length; loop++) {
			ggcode[loop] = ggcode[loop + 1];
		}
		
		ggcode[--(*length)] = 0;
	} else if (*length != 8) {
		return -1;
	}
	
	for (loop = 0; loop < *length; loop++) {
		if ((ggcode[loop] = decodeSNEStoHex(ggcode[loop])) == 0) {
			return -2;
		}
	}
	
	return 1;
}

void decodeSNES(unsigned long int cipher, unsigned short int *value, unsigned long int *address) {
	unsigned long int segment, decrypted = 0;
	
	segment = (cipher & 0x00003C00) << 10;
	decrypted |= segment;
	
	segment = (cipher & 0x0000003C) << 14;
	decrypted |= segment;
	
	segment = (cipher & 0x00F00000) >> 8;
	decrypted |= segment;
	
	segment = (cipher & 0x00000003) << 10;
	decrypted |= segment;
	
	segment = (cipher & 0x0000C000) >> 6;
	decrypted |= segment;
	
	segment = (cipher & 0x000F0000) >> 12;
	decrypted |= segment;
	
	segment = (cipher & 0x000003C0) >> 6;
	decrypted |= segment;
	
	*address = decrypted;
	*value = (unsigned short int)(cipher >> 24);
}

void convertSNES(char *ggcode) {
	short int length = strlen(ggcode), result;
	char saveCode[13],str[30],*romstr = "ROM Address: %06lXh";
	unsigned long int cipher, cpu;
	unsigned short int value;
	HANDLE dlg = H_NULL;
	
	// preserve original ggcode
	strcpy(saveCode,ggcode);
	
	result = isValidSNES(ggcode,&length);
	
	if (result == -2) {
		DlgMessage((char *)errors[INVALID_CODE],"Only 0123456789ABCDEF are valid!",BT_OK,BT_NONE);
	} else if (result == -1) {
		DlgMessage((char *)errors[INVALID_LENGTH],"SNES Codes must be 8 characters!",BT_OK,BT_NONE);
	} else {
		if ((dlg = DialogNewSimple(SNES_WIDTH,SNES_HEIGHT)) == H_NULL) {
			DlgMessage((char *)errors[MEMORY_ERROR],(char *)errors[NO_FREE_MEMORY],BT_OK,BT_NONE);
		} else {
			cipher = toDecimal(ggcode,16);
			decodeSNES(cipher,&value,&cpu);
			
			DialogAddTitle(dlg,titles[SNES],BT_OK,BT_NONE);
			
			sprintf(str,ggstr,saveCode);
			DialogAddText(dlg,5,15,str);
			
			sprintf(str,"Value: %02hXh, CPU Address: %06lXh", value, cpu);
			DialogAddText(dlg,5,25,str);
			
			if ((cpu >= 0x400000 && cpu <= 0x7FFFFF) || (cpu >= 0xC00000 && cpu <= 0xFFFFFF)) {
				// Hi-ROM
				sprintf(str,romstr,(cpu > 0xBFFE00) ? ((cpu - 0xC00000) + 0x200) : ((cpu - 0x400000) + 0x200));
			} else {
				// Lo-ROM
				sprintf(str,romstr,((cpu & 0x7FFF) | ((cpu & 0xFF0000) >> 1)) + 0x200);
			}
			
			DialogAddText(dlg,5,35,str);
			DialogDo(dlg,CENTER,CENTER,NULL,NULL);
			
			HeapFree(dlg);
		}
	}
}

#endif
